@@ -6,7 +6,7 @@ use Apache::Table ();
 
 {
     no strict;
-    $VERSION = '0.3004';
+    $VERSION = '0.3103';
     @ISA = qw(Apache);
     __PACKAGE__->mod_perl::boot($VERSION);
 }
@@ -26,6 +26,17 @@ sub param {
     return wantarray ? ($tab->get($name)) : scalar $tab->get($name);
 }
 
+sub instance {
+    my $class = shift;
+    my $r = shift;
+    if (my $apreq = $r->pnotes('apreq')) {
+        return $apreq;
+    }
+    my $new_req = $class->new($r, @_);
+    $r->pnotes('apreq', $new_req);
+    return $new_req;
+}
+
 1;
 __END__
 
@@ -43,13 +54,13 @@ Apache::Request - Methods for dealing with client request data
 I<Apache::Request> is a subclass of the I<Apache> class, which adds methods
 for parsing B<GET> requests and B<POST> requests where I<Content-type>
 is one of I<application/x-www-form-urlencoded> or 
-I<multipart/form-data>.
+I<multipart/form-data>. See the libapreq(3) manpage for more details.
 
 =head1 Apache::Request METHODS
 
 =over 4
 
-=item new
+=head2 new
 
 Create a new I<Apache::Request> object with an I<Apache> request_rec object:
 
@@ -91,7 +102,33 @@ error code if a file upload is attempted:
 
 =back
 
-=item parse
+=head2 instance
+
+The instance() class method allows Apache::Request to be a singleton.
+This means that whenever you call Apache::Request->instance() within a
+single request you always get the same Apache::Request object back.
+This solves the problem with creating the Apache::Request object twice
+within the same request - the symptoms being that the second
+Apache::Request object will not contain the form parameters because
+they have already been read and parsed.
+
+  my $apr = Apache::Request->instance($r, DISABLE_UPLOADS => 1);
+
+Note that C<instance()> call will take the same parameters as the above
+call to C<new()>, however the parameters will only have an effect the
+first time C<instance()> is called within a single request. Extra
+parameters will be ignored on subsequent calls to C<instance()> within
+the same request.
+
+Subrequests receive a new Apache::Request object when they call
+instance() - the parent request's Apache::Request object is not copied
+into the subrequest.
+
+Also note that it is unwise to use the C<parse()> method when using
+C<instance()> because you may end up trying to call it twice, and
+detecting errors where there are none.
+
+=head2 parse
 
 The I<parse> method does the actual work of parsing the request.
 It is called for you by the accessor methods, so it is not required but
@@ -107,7 +144,7 @@ occur:
 	return $status; 
     } 
 
-=item param
+=head2 param
 
 Get or set request parameters:
 
@@ -116,7 +153,7 @@ Get or set request parameters:
     my @params = $apr->param;
     $apr->param('foo' => [qw(one two three)]);
 
-=item upload
+=head2 upload
 
 Returns a single I<Apache::Upload> object in a scalar context or
 all I<Apache::Upload> objects in an array context: 
@@ -140,19 +177,19 @@ object associated with the given name:
 
 =over 4
 
-=item name
+=head2 name
 
 The name of the filefield parameter:
 
     my $name = $upload->name;
 
-=item filename
+=head2 filename
 
 The filename of the uploaded file:
 
     my $filename = $upload->filename;
 
-=item fh
+=head2 fh
 
 The filehandle pointing to the uploaded file:
 
@@ -161,13 +198,13 @@ The filehandle pointing to the uploaded file:
 	...
     }
 
-=item size
+=head2 size
 
 The size of the file in bytes:
 
     my $size = $upload->size;
 
-=item info
+=head2 info
 
 The additional header information for the uploaded file.
 Returns a hash reference tied to the I<Apache::Table> class.
@@ -181,7 +218,7 @@ a given header rather than a hash reference.  Examples:
 
     my $val = $upload->info("Content-type");
 
-=item type
+=head2 type
 
 Returns the I<Content-Type> for the given I<Apache::Upload> object:
 
@@ -189,7 +226,7 @@ Returns the I<Content-Type> for the given I<Apache::Upload> object:
     #same as
     my $type = $upload->info("Content-Type");
 
-=item next
+=head2 next
 
 As an alternative to using the I<Apache::Request> I<upload> method in
 an array context:
@@ -206,6 +243,10 @@ an array context:
 
 =back
 
+=head1 SEE ALSO
+
+libapreq(3)
+
 =head1 CREDITS
 
 This interface is based on the original pure Perl version by Lincoln Stein.
